
> module GameState where
> import Robot
> import Map
> import List
> import Maybe
> import Paths

Things we need to keep track of:

The map
our money
our capacity
The state of all robots
  - packages carried
  - position
  - movements
The state of all packages
  - destination
  - weight
  - location


Some of these facts may be unknown

> data GameState = GameState Map
>                            [PackageData]
>                            [RobotData]
>                            Int -- our money
>                            Int -- our capacity
>                            Int -- our ID
>		             [Position] -- Bases that may not be empty
>		             [Int] -- Robots alive
>                            Int -- our score
>		             AllMultiPaths -- all possible routes
>    deriving Show

> data PackageData = PackageData Int -- id
>                                (Maybe Position) -- destination
>                                (Maybe Int) -- weight
>                                PackageLocation
>    deriving Show
> instance Eq PackageData where
>    p1 == p2 = (packageId p1) == (packageId p2)

> data PackageLocation = PackageOnFloor Position
>                      | PackageInRobot Int
>    deriving (Show, Eq)

> packagePos (PackageData _ _ _ (PackageOnFloor p)) = p
> packagePos (PackageData _ _ _ (PackageInRobot _)) = error "Package is being carried!"
> packageDest (PackageData _ d _ _) = d
> packageWeight (PackageData _ _ w _) = w

> packageId (PackageData i _ _ _) = i

> myPackages g = packagesWith g (getId g)

> packagesWith g ident = [packageId p|p<-pl, inside ident p]
>     where pl = getPackageDatas g
>           inside id (PackageData _ _ _ (PackageOnFloor _)) = False
>           inside id (PackageData _ _ _ (PackageInRobot i)) = i==id

> packagesAt g pos = [packageId p|p<-pl, (ps p) == (PackageOnFloor pos)]
>     where pl = getPackageDatas g
>           ps (PackageData _ _ _ l) = l

> data RobotData = RobotData Int -- id
>                            Position -- location
>    deriving Show
> instance Eq RobotData where
>    r1 == r2 = (robotId r1) == (robotId r2)

> robotId (RobotData i _) = i
> robotPos (RobotData _ p) = p

> myPosition g =  robotPos (getRobotData g (getId g))

> availableCapacity g = (getCapacity g) - (foldl (+) 0 (map wt (myPackages g)))
>     where wt id = w
>               where (PackageData _ _ (Just w) _) = getPackageData g id



> getAllPaths (GameState _ _ _ _ _ _ _ _ _ l) = l

> getRobotDatas (GameState _ _ l _ _ _ _ _ _ _) = l

> getRobotData (GameState _ _ l _ _ _ _ _ _ _) id = let rs = [r | r<-l, (robotId r)==id]
>                                         in case rs of
>                                              [] -> error ("Nonexistent robot ID "++(show id))
>                                              [x] -> x
>                                              _ -> error "Duplicate robots"

> setRobotData (GameState m pl rl money cap id base life s pth) r =
>        GameState m pl l money cap id base life s pth
>     where l = r:[b | b<-rl, b/=r]

> packageExists (GameState _ l _ _ _ _ _ _ _ _) id = any (\x->(packageId x)==id) l

> getPackageDatas (GameState _ l _ _ _ _ _ _ _ _) = l

> getPackageData (GameState _ l _ _ _ _ _ _ _ _) id = let rs = [p | p<-l, (packageId p)==id]
>                                           in case rs of
>                                              [] -> error ("Nonexistent package ID "++(show id))
>                                              [x] -> x
>                                              _ -> error "Duplicate package IDs"

> setPackageData (GameState m pl rl money cap id base life s pth) p =
>        GameState m l rl money cap id base life s pth
>     where l = p:[b | b<-pl, b/=p]

> deletePackageData (GameState m pl rl money cap id base life s pth) p =
>        GameState m l rl money cap id base life s pth
>     where l = [b | b<-pl, (packageId b)/=p]

> deletePackagesAt (GameState m pl rl money cap id base life s pth) pos =
>        GameState m l rl money cap id base life s pth
>     where l = [b | b<-pl, not (packageIsAt b pos)]
>           packageIsAt (PackageData _ _ _ (PackageOnFloor p)) pos | p==pos = True
>           packageIsAt _ _ = False

> getMoney (GameState _ _ _ money _ _ _ _ _ _) = money
> setMoney (GameState m pl rl _ cap id base life s pth) money = 
>     GameState m pl rl money cap id base life s pth
> addMoney gs diff = setMoney gs ((getMoney gs)+diff)

> getScore (GameState _ _ _ _ _ _ _ _ s _) = s
> addScore (GameState a b c d e f g h s pth) si =
>     GameState a b c d e f g h (s+si) pth

> getCapacity (GameState _ _ _ _ cap _ _ _ _ _) = cap

> getMap (GameState m _ _ _ _ _ _ _ _ _) = m

> getId (GameState _ _ _ _ _ id _ _ _ _) = id 

> getBases (GameState _ _ _ _ _ _ base _ _ _) = base

> getRobotList (GameState _ _ rl _ _ _ _ _ _ _) = rl



> updateGameState g (RobotMoved id dir) = 
>           if dead then (removeLife g' id) else g'
>     where (RobotData _ pos) = getRobotData g id
>           rm = RobotData id pos'
>	    pos' = (move dir pos)
>	    g' = setRobotData g rm
>           dead = (floorAt (getMap g) pos')==Water
> updateGameState g (RobotSentTo id npos) = setRobotData g (RobotData id npos)
> updateGameState g (RobotPickedUp id pid) = 
>      if packageExists g pid
>      then setPackageData g pm
>      else setPackageData g (PackageData pid Nothing Nothing (PackageInRobot id))
>     where (PackageData _ pos wt _) = getPackageData g pid
>           pm = PackageData pid pos wt (PackageInRobot id)
> updateGameState g (RobotDropped id pid) = 
>    if packageExists g pid
>    then if (isJust dest) && (fromJust dest)==pos 
>         then if id == (getId g) 
>              then deletePackageData (addScore g (fromJust $ packageWeight $ getPackageData g pid)) pid
>              else deletePackageData g pid
>         else setPackageData g pm
>    else setPackageData g (PackageData pid Nothing Nothing (PackageOnFloor pos))
>   where (PackageData _ dest wt _) = getPackageData g pid
>         pm = PackageData pid dest wt (PackageOnFloor pos)
>         pos = (robotPos (getRobotData g id))
> updateGameState (GameState a b c d e f g h j pth) (RobotAlive i) = 
>     GameState a b c d e f g (i:h) j pth
> removeLife (GameState a b c d e f g life h pth) id =
>     GameState a b c d e f g (delete id life) h pth


> flushDeadRobots (GameState a pl rs d e f g life h pth) =
>     GameState a npl quick d e f g [] h pth
>   where (quick, dead) = partition alive rs
>         alive r = any (\n->i==n) life
>             where i = robotId r
>         npl = filter (\p->not (any (inRobot p) dead)) pl
>         inRobot::PackageData->RobotData->Bool
>         inRobot (PackageData _ _ _ (PackageInRobot i)) r | i==(robotId r) = True
>         inRobot _ _ = False
> removeBaseHere (GameState a b c d e f base h j pth) pos =
>     GameState a b c d e f (filter (\pos' -> pos/=pos') base) h j pth

> learnPackages g l = lP (deletePackagesAt g pos) l
>     where lP g [] = g
>           lP g (x:xs) = lP (setPackageData g p) xs
>             where (id, dest, wt) = x
>                   p = PackageData id (Just dest) (Just wt) (PackageOnFloor pos)
>           pos = myPosition g

> robotAt gs pos =
>     case rl of
>      [] -> Nothing
>      (x:_) -> Just x
>  where rl = [r | r<-rd, (robotPos r)==pos]
>        rd = getRobotDatas gs



> describe g = "Robot "++(show (getId g))++" at "++(show (myPosition g))++
>              "\ncarrying packages ["++(unwords $ map show $ myPackages g)++"]"++
>              "\nremaining carrying capacity "++(show $ availableCapacity g)++
>              " remaining money "++(show $ getMoney g)++"\n"++
>              "score "++(show $ getScore g)++
>              "\n"
