> module MemoryBot where
> import SimplerRobotG
> import GameState
> import Map
> import Robot
> import List
> import SimplerRobot
> import Paths
> import RobotAlgos
> import Array
> import StdSugg
> import Maybe

> import Debug.Trace

> data (Eq a) => MemoryState a = MemoryState GameState [a] deriving Show

> getGameState :: (Eq a) => MemoryState a -> GameState
> getGameState (MemoryState g _) = g

> updateGameStateM :: (Eq a) => MemoryState a -> GameState -> MemoryState a
> updateGameStateM (MemoryState _ a) g = MemoryState g a

> rememberThis :: (Eq a) => MemoryState a -> [a] -> MemoryState a
> rememberThis (MemoryState g _) d = MemoryState g d

> recall :: (Eq a) => MemoryState a -> [a]
> recall (MemoryState _ d) = d

> initMemoryRobotG:: (Eq a) => Map->(Int,Int,Int)->MemoryState a
> initMemoryRobotG m d = MemoryState
>		   (initSimplerRobotG m d)
>	           []

> memoryBotUpdate1 remember ms events pkgs =
>     (remember gs (updateGameStateM ms gs') events pkgs)
>  where
>   gs = getGameState ms
>   gs' = simplerRobotUpdate1G gs events pkgs
>   

> memoryBotUpdate2 remember ms rc =
>     (remember gs (updateGameStateM ms gs') rc)
>  where
>   gs = getGameState ms
>   gs' = simplerRobotUpdate2G gs rc

> memoryBotChoiceHelper remember moves notmoves paths id memState =
>     (remember memState reason 
>       (if null sugg then Pick 1 id []
>        else addCost (moveAway-(dontMove best)) best))
>  where
>     state = getGameState memState
>     sugg = (mergeSortedListsBy cmp
>      (map sortSugg moves))
>     sortSugg (Suggestion s) = (sortBy revcmp (s state paths))
>     cmp (_,x,_) (_,y,_) = (compare x y)
>     revcmp x y = (cmp y x)
>     filterSugg = (filter (\p ->
>                   (any (\r -> (checkRestrict r p)) notmoves)) sugg)
>     checkRestrict (Restriction r) (a,b,_) = (fst (r state paths (moveToDir a))) < b
>     (best,valBest,reason) = if null filterSugg then head sugg else head filterSugg
>     moveAway = if null notmoves then 0 else
>        maximum [(snd (r state paths Nothing)) | (Restriction r)<-notmoves]
>     dontMove (Move _ _ d) = if null notmoves then 0 else
>        maximum [(snd (r state paths (Just d))) | (Restriction r)<-notmoves]
>     dontMove _ = 0

> memoryRobotChoice remember suggs restr ms =
>   if safe then setCost 1 choice else choice
>  where
>   gs = getGameState ms
>   paths = if (mx*my*(min (mx*my) (getMoney gs))) < 30000000
>           then getAllPaths gs
>           else (findAllMultiPaths (wallsFromMap map))
>   map = getMap gs
>   choice = memoryBotChoiceHelper remember suggs restr paths (getId gs) ms
>   safe = null (whoCanPushMe gs (getId gs))
>   (_,(mx,my))= bounds map
>   setCost a (x,(Move _ b c)) = (x,(Move a b c))
>   setCost a (x,(Pick _ b c)) = (x,(Pick a b c))
>   setCost a (x,(Drop _ b c)) = (x,(Drop a b c))

> doMRobot update1 update2 choice state events pkgs = (state''', decision)
>  where
>   state' = update1 state events pkgs
>   (state'',decision) = choice state'
>   state''' = update2 state'' decision

> botM rem1 rem2 rem3 suggs restr = (initMemoryRobotG,
>        (doMRobot (memoryBotUpdate1 rem1) (memoryBotUpdate2 rem3)
>         (memoryRobotChoice rem2 suggs restr)),
>        (\ms -> describe (getGameState ms)))

> noLoopRem1 _ m _ _ = (m::MemoryState (Position,Maybe Reason))

> noLoopRem3 _ m _ = (m::MemoryState (Position,Maybe Reason))

> noLoopRem2 m reason cmd = if size /= 12 then (m2, cmd) else 
>   if anyLoop && not (null forgetable) then (m1, (Pick 1 id [])) else (m2, cmd)
>  where
>   id = getId gs
>   gs = getGameState m
>   myPos = myPosition gs
>   stack = recall m
>   newStack = {-(\x-> trace (show x) x) $-} take 12 ((myPos,reason):stack)
>   size = length newStack
>   forgetable = (filter (\(x,y) -> isJust y) newStack)
>   anyLoop = or [loopCheck n newStack | n<-[4,6]]
>   gs' = forget gs (fromJust (snd (head forgetable)))
>   m1 = (rememberThis (updateGameStateM m gs') [])
>   m2 = (rememberThis m newStack)

> noLoopBot :: (Map -> (Int, Int, Int) -> MemoryState (Position,Maybe Reason),
>	MemoryState (Position,Maybe Reason)
>	-> [ServerEvent]
>	-> [(Int, Position, Int)]
>	-> (MemoryState (Position,Maybe Reason), RobotCommand),
>	MemoryState (Position,Maybe Reason) -> [Char])

> noLoopBot = (botM noLoopRem1 noLoopRem2 noLoopRem3 
>       prioritySugg avoidRest)

> loopCheck :: (Eq a) => Int -> [a] -> Bool
> loopCheck n [] = False
> loopCheck n l = all (\e -> e==hd) tl
>  where
>   (hd:tl) = group l
>   group [] = []
>   group ls | length ls < n = [ls]
>	     | otherwise = let (l1, l2) = splitAt n ls in (l1:(group l2))

I should avoid forgetting bases if possible

> forget gs (PD pk) = 
>     deletePackageData gs (packageId pk)
> forget gs (RD bot) = foldl (\gs pkId -> deletePackageData gs pkId) gs
>	       (packagesWith gs (robotId bot))
> forget gs (HB pos) = 
>     removeBaseHere gs pos


