
> module Paths where

> import GHC.Arr
> import List
> import Map
> import Debug.Trace

> import qualified Data.Array.ST
> import Control.Monad.ST

> filterNothing ((Just x):xs) = x:(filterNothing xs)
> filterNothing (Nothing:xs) = filterNothing xs
> filterNothing [] = []

> data Range = Range Int Direction
>            | Unreachable
>     deriving Show

> compareRange (Range n1 d1) r@(Range n2 d2) | n1<=n2 = Nothing
>                                            | True   = Just r
> compareRange Unreachable r@(Range _ _) = Just r

> passable walls p = (inside walls p) && (walls!p)



> wallsFromMap m = amap (\x -> x==Plain || x==Base) m

> wallsFromString s = array ((1,1),(mx,my)) 
>                           [( (i,j),convert (s!!(i-1+(mx+1)*(j-1))) ) | i<-[1..mx],
>                                                                        j<-[1..my]]
>    where mx = case elemIndex '\n' s of Just x  -> x
>                                        Nothing -> error "No newline found!"
>          my = (length s) `div` (mx+1)
>          convert c = case c of
>                             '.' -> True
>                             '~' -> False
>                             '#' -> False
>                             '@' -> True




> writeValues a ((i,v):xs) = do writeSTArray a i v
>                               writeValues a xs
> writeValues a [] = do return ()

> type Paths = (Array (Int, Int) Range)

> findPaths::(Array (Int,Int) Bool)->(Int,Int)->Paths
> findPaths walls pos = runST (do
>    ranges <- (Data.Array.ST.newArray ((bounds walls)::((Int,Int),(Int,Int))) Unreachable)
>    writeValues ranges initialRanges
>    spread ranges [p|(p,_)<-initialRanges]
>    writeSTArray ranges pos (Range 0 North)
>    freezeSTArray ranges)
>  where initialRanges = (filter (\ (p, _) -> passable walls p) 
>                                (map (\d -> (move d pos, Range 1 d)) [North, South, East, West]))
>        spread ranges [] = do return ()
>        spread ranges (x:xs) = do na <- explore ranges x
>                                  spread ranges (xs++na) -- breadth-first search
>        explore ranges p = do (Range n d) <- readSTArray ranges p
>                              n <- sequence (map (examine ranges (Range (n+1) d)) 
>                                                 [move North p,
>                                                  move South p,
>                                                  move East p,
>                                                  move West p])
>                              return (filterNothing n)
>        examine ranges r p = do (if (not (passable walls p))
>                                 then (return Nothing)
>                                 else do er <- readSTArray ranges p
>                                         case compareRange er r of
>                                              Nothing -> return Nothing
>                                              Just rg -> do writeSTArray ranges p rg
>                                                            return (Just p))




> data MultiRange = MultiRange Int [Direction]
>                 | MultiUnreachable
>     deriving Show

FIXME: merging lists ought to be faster, maybe use arrays

> mergeLists [] l = l
> mergeLists l [] = l
> mergeLists l1 l2 | l1==l2 = l1
> mergeLists (x:xs) l = if any (\y->y==x) l then mergeLists xs l else x:(mergeLists xs l)

> compareMultiRange r1@(MultiRange n1 d1) r2@(MultiRange n2 d2) | n1<n2 = (False, r1)
>                                                               | n1==n2 = (False, (MultiRange n1 (mergeLists d1 d2)))
>                                                               | n1>n2 = (True, r2)
> compareMultiRange MultiUnreachable r@(MultiRange _ _) = (True, r)
> compareMultiRange r MultiUnreachable = (False, r)



> type MultiPaths = (Array (Int, Int) MultiRange)

> findMultiPaths::(Array (Int,Int) Bool)->(Int,Int)->MultiPaths
> findMultiPaths walls pos = runST (do
>    ranges <- (Data.Array.ST.newArray ((bounds walls)::((Int,Int),(Int,Int))) MultiUnreachable)
>    writeValues ranges initialRanges
>    spread ranges [p|(p,_)<-initialRanges]
>    (MultiRange _ ds) <- readSTArray ranges pos
>    writeSTArray ranges pos (MultiRange 0 ds)
>    freezeSTArray ranges)
>  where initialRanges = (filter (\ (p, _) -> passable walls p) 
>                                (map (\d -> (move d pos, MultiRange 1 [d])) [North, South, East, West]))
>        spread ranges [] = do return ()
>        spread ranges (x:xs) = do na <- explore ranges x
>                                  spread ranges (xs++na) -- breadth-first search
>        explore ranges p = do (MultiRange n d) <- readSTArray ranges p
>                              n <- sequence (map (examine ranges (MultiRange (n+1) d)) 
>                                                 [move North p,
>                                                  move South p,
>                                                  move East p,
>                                                  move West p])
>                              return (filterNothing n)
>        examine ranges r p = do (if (not (passable walls p))
>                                 then (return Nothing)
>                                 else do er <- readSTArray ranges p
>                                         case compareMultiRange er r of
>                                              (b, rg) -> do writeSTArray ranges p rg
>                                                            return (if b then Just p else Nothing))



> directionTo::Paths -> (Int,Int) -> Maybe Direction
> directionTo path p2 = case path!p2 of
>	Unreachable -> Nothing
>    	(Range _ d) -> Just d
> distanceTo::Paths -> (Int,Int) -> Maybe Int 
> distanceTo path p2 = case path!p2 of
>	Unreachable -> Nothing
>    	(Range d _) -> Just d

> directionsTo::MultiPaths -> (Int,Int) -> [Direction]
> directionsTo path p2 = case path!p2 of
>	MultiUnreachable -> [] -- can't get there from here
>    	(MultiRange _ d) -> d
> distanceToM::MultiPaths -> (Int,Int) -> Maybe Int 
> distanceToM path p2 = case path!p2 of
>	MultiUnreachable -> Nothing
>    	(MultiRange d _) -> Just d




> type AllPaths = Array (Int,Int) Paths

> findAllPaths :: (Array (Int,Int) Bool)->AllPaths
> findAllPaths a = (array b [(x,(findPaths a x)) | x<-(range b)])
>	   where b = (bounds a)

> type AllMultiPaths = Array (Int,Int) MultiPaths

> findAllMultiPaths :: (Array (Int,Int) Bool)->AllMultiPaths
> findAllMultiPaths a = (array b [(x,(findMultiPaths a x)) | x<-(range b)])
>	   where b = (bounds a)



