> module SimplerRobotG where
> import SimplerRobot
> import Paths
> import GameState
> import Map
> import StdSugg
> import StdRest
> import Robot
> import RobotAlgos
> import Array

> initSimplerRobotG::Map->(Int,Int,Int)->GameState
> initSimplerRobotG map (id,cap,funds) = GameState map
>                                                [] -- no known packages
>                                                [] -- no known robots (not even us! whee!)
>                                                funds
>                                                cap
>                                                id
>                                                (findBases map)
>                                                [] -- no known living robots
>                                                0 -- score is zero initially
>					         (findAllMultiPaths (wallsFromMap map))

Two options for getting the paths.  The choice should prehaps depend on how big the map is?
Prehas see if mx*my*(min funds mx*my) is past a certain threashold like 30 000 000

> simplerRobotChoiceG suggs restr gs =
>   if safe then setCost 1 choice else choice
>  where
>   paths = if False && (mx*my*(min (mx*my) (getMoney gs))) < 30000000
>           then getAllPaths gs
>	    else (findAllMultiPaths (wallsFromMap map))
>   map = getMap gs
>   choice = simplerRobotChoice suggs restr paths (getId gs) gs
>   safe = null (whoCanPushMe gs (getId gs))
>   (_,(mx,my))= bounds map

> simplerRobotUpdate1G gs events pkgs = removeBaseHere gs' (myPosition gs')
>     where gs' = flushDeadRobots $ learnPackages (foldl updateGameState gs events) pkgs
>          

> cost (Move b _ _) = abs b
> cost (Pick b _ _) = abs b
> cost (Drop b _ _) = abs b

> setCost a (Move _ b c) = (Move a b c)
> setCost a (Pick _ b c) = (Pick a b c)
> setCost a (Drop _ b c) = (Drop a b c)

> simplerRobotUpdate2G gs rc = addMoney gs (-(cost rc))

> botG choice = (initSimplerRobotG, 
>	 (doSRobot simplerRobotUpdate1G simplerRobotUpdate2G choice),
>	 describe)

> newtype ReverseValue = ReverseValue Int deriving (Show, Eq)
> instance Ord ReverseValue where
>     compare (ReverseValue a) (ReverseValue b) =
>                   case (compare a b) of
>                              LT -> GT
>                              GT -> LT
>                              EQ -> EQ

> lazyBot = botG (simplerRobotChoiceG ([]::[Suggestion GameState () ()])
>	  ([]::[Restriction GameState ()]))

> hunterValueHunt (rc, val, res) = (rc,(1,ReverseValue val), res)
> hunterValueSeekBase (rc, val, res) = (rc,(0,ReverseValue val), res)

> hunterBot = botG (simplerRobotChoiceG 
>	    [Suggestion (\a b -> (map hunterValueHunt (huntSugg 10 a b))),
>            Suggestion (\a b -> (map hunterValueSeekBase (goToBaseSugg 10 a b)))
>           ]
>	    [])

> newtype PriorityValue = PriorityValue (Int,Int, Int) deriving (Show, Eq)
> instance Ord PriorityValue where
>     compare (PriorityValue (x,a,b)) (PriorityValue (y,c,d)) =
>                    if (x < y) then GT
>		     else if (y < x) then LT 
>		     else if (a < c) then GT
>		     else if (c < a) then LT
>		     else if (b < d) then GT
>		     else if (d < b) then LT else EQ

> priorityValueHunt (rc, val, res) = (rc, (PriorityValue (0,4, val)), Just res)
> priorityValuePickUp rc = (rc, (PriorityValue (0, 2, 0)), Nothing)
> priorityValueDropOff rc = (rc, (PriorityValue (0, 0, 0)), Nothing)
> priorityValueGoToBase (rc, val, res) = (rc, (PriorityValue (0, 3, (3*val))), Just res)
> priorityValueFetch (rc, val, res) = (rc, (PriorityValue (0, 3,(2*val))), Just res)
> priorityValueDeliver (rc, val, res) = (rc, (PriorityValue (0, 3,val)), Just res)

> priorityValueDeath True = (PriorityValue (-1,0,0), 50)
> priorityValueDeath False = (PriorityValue (1,0,0), 0)

> coreSugg =
>           [Suggestion (\a b -> 
>		 (map priorityValueHunt (huntSugg cost a b)))
>           ,Suggestion (\a b -> 
>		 (map priorityValuePickUp (pickUpSugg cost a b)))
>           ,Suggestion (\a b -> 
>		 (map priorityValueDropOff (dropOffSugg cost a b)))
>           ,Suggestion (\a b ->
>                (map priorityValueGoToBase (goToBaseSugg cost a b)))
>           ,Suggestion (\a b ->
>                (map priorityValueFetch (fetchSugg cost a b)))
>           ,Suggestion (\a b ->
>                (map priorityValueDeliver (deliverSugg cost a b)))]
>  where 
>   cost = 10

> prioritySugg = ((Suggestion (\a b ->
>                (map priorityValueHunt (huntSugg cost a b)))):coreSugg)
>  where 
>   cost = 10

> priorityRest = [Restriction (\a b c ->
>		       (priorityValueDeath (deathRest a b c)))]

> priorityBot = botG (simplerRobotChoiceG prioritySugg priorityRest)

> avoidValuePush True = (PriorityValue (0,3,0), 3)
> avoidValuePush False = (PriorityValue (1,0,0),0)

>  where
>   cost = 10

> avoidRest = ((Restriction (\a b c ->
>                (avoidValuePush (pushRest a b c)))):priorityRest)

> avoidBot = botG (simplerRobotChoiceG prioritySugg avoidRest)



